<?php
/*
Plugin Name: BW_Registration
Plugin URI: http://bewide.it/
Description: BW Registration plugin.
Version: 1.0
Author: Nicolai Nita [BeWide]
Author URI: http://bewide.com/
License: BeWide
*/

$BW_Registration = new BW_Registration();

class BW_Registration {

	var $Admin;

	function BW_Registration() {

		// Initialization
		$this->define_vars();
		register_activation_hook( __FILE__, array($this, 'activation') );
		
		if(is_admin()) {
		} else {

			$this->register_hooks();
		}
	}

	function define_vars() {
		global $wpdb;

		define("BW_REGISTRATION_URL", WP_PLUGIN_URL . "/bw_registration");
		define("BW_REGISTRATION_DIR", dirname(__FILE__));
		define("BW_REGISTRATION_PREFIX", "bf_");

		$wpdb->bf_users = BW_REGISTRATION_PREFIX . 'users';

		require_once BW_REGISTRATION_DIR . "/common/defined.php";
		require_once BW_REGISTRATION_DIR . "/core/users.php";
		require_once BW_REGISTRATION_DIR . "/core/authentication.php";
		require_once BW_REGISTRATION_DIR . "/common/users_func.php";
	}

	/**
	 * Plugin activation
	 *
	 * @global <type> $wp_version
	 * @global <type> $wpdb
	 */
	function activation() {
		global $wp_version, $wpdb;

		require_once(BW_REGISTRATION_DIR . '/core/activation.php');
	}

	/**
	 * Admin notices
	 */
	function admin_notices() {

	}

	/**
	 * Register hooks
	 */
	function register_hooks() {
		add_action( 'init', array($this, 'register_user') );

		add_action( 'init', array($this, 'password_reset') );

		add_action( 'init', array($this, "login_user") );

		add_action( 'comment_post', array($this, "on_comment") );

	}

	function on_comment($comment_id) {
		global $wpdb;
		
		if(is_user_logged_in()) {
			return true;
		}
		
		$user = AuthenticationManager::get_logged_user();
		
		if (empty($user)) {
			$query = "DELETE FROM `$wpdb->comments` WHERE `comment_ID` = %d";
			$wpdb->query( $wpdb->prepare($query, $comment_id) );

		} else {
			
			$query = "UPDATE `$wpdb->comments` SET `user_id` = %d WHERE `comment_ID` = %d";
			$wpdb->query( $wpdb->prepare($query, $user->id, $comment_id) );
		}
	}

	function register_user() {

		$user_type = isset($_POST['user']['doctor']) ? UserManager::TYPE_MEDIC : UserManager::TYPE_USER;

		$_POST['user']['user_type'] = $user_type;
		
		if (isset($_POST['register_user']) && isset($_POST['user'])
				&& UserManager::checkForRegistration((object) $_POST['user'])
				&& UserManager::confirmPassword($_POST['user']['password'], $_POST['user']['password-confirm'], $user_type)) {

			if ($user_type == UserManager::TYPE_USER)
				unset($_POST['user']['specialization']);

			UserManager::register((object) $_POST['user']);
			
		} else if (!empty($_GET['emailconfirm']) && !empty($_GET['key'])) {
			
			UserManager::activate_user($_GET['emailconfirm'], $_GET['key']);
		}
	}

	function login_user() {

		if (isset($_GET['logout'])) {
			
			AuthenticationManager::un_authenticate();
		} else if (isset($_POST['user']['login']) && $_POST['user']['login']) {

			if (AuthenticationManager::authenticate($_POST['user']['email'], $_POST['user']['password'])) {
				
				wp_redirect(home_url("/login/"));
				die();
			}
		}
	}

	function password_reset() {
		if (isset($_POST['password_reset']) && isset($_POST['user']['email']) && isset($_POST['user']['password'])) {
			
			if (empty($_POST['user']['email']) || empty($_POST['user']['password']) || empty($_POST['user']['password_confirm'])) {

				UserManager::setErrorMessage("I campi contrassegnati con * sono obbligatori.");
				return false;
			}
			
			if (!UserManager::checkEmail($_POST['user']['email'])) {

				UserManager::setErrorMessage("ATTENZIONE: l’indirizzo e-mail inserito non è corretto.");
				return false;
			}

			if (strcmp($_POST['user']['password'], $_POST['user']['password_confirm']) !== 0) {

				UserManager::setErrorMessage("ATTENZIONE: le password inserite non corrispondono.");
				return false;
			}

			if (strlen($_POST['user']['password']) < 4) {

				UserManager::setErrorMessage("La password deve avere almeno 4 caratteri.");
				return false;
			}

			$reset_pass_key = UserManager::register_reset_password($_POST['user']['email'], $_POST['user']['password']);
			UserManager::send_reset_password_register_confirm($_POST['user']['email'], $reset_pass_key);

			UserManager::setMessage("Ti abbiamo appena inviato una mail. Ti preghiamo di leggere la tua posta e confermare la tua iscrizione cliccando sul link contenuto nel messaggio.");
			
		} else if (isset($_GET['resetpassemailconfirm'])) {

			$email = $_GET['resetpassemailconfirm'];

			if (!UserManager::userExists($email)) {

				UserManager::setMessage(sprintf("L’indirizzo e-mail inserito non è presente nel nostro database.
					<br />
					<br />
					Prova con un altro indirizzo o registrati di nuovo. <br />%sRegistrati%s", $email, '<a href="'.home_url('register/').'">', '</a>'));
				return false;
			}

			$result = UserManager::confirm_reset_password($email, $_GET['key']);

			if ($result) {
				UserManager::setMessage(sprintf("La tua nuova password e' stata confermata. Vai alla pagina %slogin%s.", '<a href="'.home_url("login").'">', '</a>'));
			} else {
				UserManager::setErrorMessage("Per modificare la tua password compila il seguente modulo.");
			}
		}
	}
}