<?php
/**
 * $user - object('name' => string,
 * 'surname'	=> string,
 * 'birthday'	=> string,
 * 'email'	=> string,
 * 'password'	=> string,
 * 'bio'	=> string)
 */
class UserManager {

	const TYPE_USER = "USER";
	
	const TYPE_MEDIC= "MEDIC";

	/* Hash string (last simbol no "Number") */
	const REG_KEY_PREFIX = "AGWJFGFXJL";
	/* Number (int) */
	const REG_KEY_OFFSET = 3573;

	private static $user_types = array('USER', 'MEDIC');
	/**
	 * Message for a register form.
	 * @var array($key => [user_type], $value => string)
	 */
	private static $msg = array();
	/**
	 * Error message for a register form.
	 * @var array($key => [user_type], $value => string)
	 */
	private static $errorMsg = array();

	function  __construct() {
		
	}

	static function register($user) {
		
		global $wpdb;

		$query = "INSERT INTO $wpdb->bf_users(`name`, `surname`, `nickname`, `email`, `password`, `user_type`, `extra`)
					VALUES(%s, %s, %s, %s, %s, %s, %s)";
		
		$wpdb->query($wpdb->prepare($query,
				$user->name,
				$user->surname,
				$user->nickname,
				$user->email,
				self::encryptPass($user->password),
				$user->user_type,
				isset($user->specialization) ? json_encode(array('specialization' => $user->specialization)) : ''
			));
		
		self::send_registration_confirmation($user, $wpdb->insert_id);
		
		self::$msg[$user->user_type] = sprintf(__('Complimenti, ti sei appena registrato a <a href="%s">botulinfree.com</a>.<br />Per attivare la registrazione ti preghiamo di seguire le istruzioni ricevute nella mail che ti abbiamo appena inviato.', "dermatoscopia"), home_url());
	}

	static function encryptPass($pass) {
		$result = md5($pass);

		return $result;
	}

	static function checkEmail($email) {
		
		return preg_match('/^[^@]+@[a-zA-Z0-9._-]+\.[a-zA-Z]+$/', $email) && true;
	}

	static function checkForRegistration($user) {
		if (empty($user)) return false;

		if (!in_array($user->user_type, self::$user_types))
				$user->user_type = self::TYPE_USER;

		$result = true;

		$result = !empty($user->name) && !empty($user->surname) && !empty($user->nickname) && !empty($user->email) && !empty($user->password);
//var_dump(!(isset($user->privacy_u) && $user->user_type == self::TYPE_USER) && !(isset($user->privacy_m) && $user->user_type == self::TYPE_MEDIC), $user);
		if ($result) {
			
			if (!self::checkEmail($user->email)) {
				
				$result = false;
				self::$errorMsg[$user->user_type] = __("La mail sbagliata.", "dermatoscopia");
			} else if (self::userExists($user->email)) {
				
				$result = false;
				self::$errorMsg[$user->user_type] = __("L'utente con la stessa mail e' stato gia' registrato.", "dermatoscopia");
			//} else if (!isset($user->privacy)) {

			//	$result = false;
			//	self::$errorMsg[$user->user_type] = __("Conferma la privacy.", "dermatoscopia");
			} else if (!(isset($user->privacy_u) && $user->user_type == self::TYPE_USER) && !(isset($user->privacy_m) && $user->user_type == self::TYPE_MEDIC)) {

				$result = false;
				self::$errorMsg[$user->user_type] = __("Per registrarsi Lei deve confermare la privacy.", "dermatoscopia");
			}
		} else {
			
			$result = false;
			self::$errorMsg[$user->user_type] = __("I campi <strong>*</strong> sono obbligatori.", "dermatoscopia");
		}

		return $result;
	}

	static function confirmPassword($pass, $confirmPass, $user_type = "USER") {

		if (!in_array($user_type, self::$user_types))
				$user_type = self::TYPE_USER;

		if (empty($pass) || empty($confirmPass)) {
			
			self::$errorMsg[$user_type] = __("You should fill the 'password' and 'retype password' fields.");
			
			return false;
		}

		$result = (strcmp($pass, $confirmPass) === 0);

		if (!$result)
			self::$errorMsg[$user_type] = __("Password doesn't match.");

		return $result;
	}

	static function userExists($email) {
		global $wpdb;

		$query = "SELECT * FROM $wpdb->bf_users WHERE `email` = %s LIMIT 1";
		$qResult = $wpdb->get_row($wpdb->prepare($query, $email));

		return !empty($qResult);
	}

	static function getMessage($user_type = "USER") {

		if (!in_array($user_type, self::$user_types))
				$user_type = self::TYPE_USER;

		if (in_array($user_type, self::$user_types) && !empty(self::$msg[$user_type]))
			return self::$msg[$user_type];
		else
			return NULL;
	}

	static function getErrorMessage($user_type = "USER") {

		if (!in_array($user_type, self::$user_types))
				$user_type = self::TYPE_USER;

		if (in_array($user_type, self::$user_types) && !empty(self::$errorMsg[$user_type]))
			return self::$errorMsg[$user_type];
		else
			return NULL;
	}

	private static function gen_reg_key($user_id) {
		$result = UserManager::REG_KEY_PREFIX . ($user_id + UserManager::REG_KEY_OFFSET);

		return $result;
	}

	private static function get_userid_by_regkey($reg_key) {
		$result = NULL;

		if (preg_match(sprintf("/%s(\d+)/i", UserManager::REG_KEY_PREFIX), $reg_key, $matches) > 0) {

			$result = $matches[1] - UserManager::REG_KEY_OFFSET;
		}

		return $result;
	}
	
	public static function send_registration_confirmation($user, $user_id) {
		
		$registration_key = self::gen_reg_key($user_id);
		$email_to = $user->email;

		$subject = sprintf("%s > %s", get_option("blogname"), __("Email di conferma", "wpzoom"));

		$activation_url = home_url("register/?emailconfirm=${email_to}&key=${registration_key}");

		$home_url = home_url();
		
		$message = <<< MESSAGE
${subject}

Ti sei appena registrato a BotulinFree.com.
Per completare la registrazione ti chiediamo di confermare il tuo indirizzo di posta elettronica cliccando sul link qui sotto::
${activation_url}

Grazie

MESSAGE;
		//var_dump($user->email, $subject, $message, sprintf("From: %s <%s>", get_option("blogname"), get_option("admin_email")));
		//return true;
		mail($user->email, $subject, $message, sprintf("From: %s <%s>", get_option("blogname"), get_option("admin_email")));

		$user_name = $user->name;
		$user_surname = $user->surname;
		$user_email = $user->email;

		$message = <<< MESSAGE
Un nuovo utente registrato:
Nome: ${user_name}
Cognome: ${user_surname}
Email: ${user_email}
MESSAGE;
		
		mail(get_option("admin_email"), "Utente registrato", $message, sprintf("From: %s <%s>", get_option("blogname"), get_option("admin_email")));
		//wp_mail($email_to, $subject, $message);
	}

	public static function activate_user($email, $reg_key) {
		global $wpdb;

		$result = false;

		$user_id = self::get_userid_by_regkey($reg_key);
		
		if ($user_id > 0) {
			$wpdb->query( $wpdb->prepare("UPDATE `$wpdb->bf_users` SET `status` = 'A' WHERE `email` = %s AND `status` = 'WFA'", $email, $user_id) );

			$result = $wpdb->rows_affected !== 0;
		}

		if (!$result)
			UserManager::$errorMsg[UserManager::TYPE_USER] = "The email is not registered in our database or is yet approved. Please register first.";
		else
			UserManager::$msg[UserManager::TYPE_USER] = 'Grazie, ora sei registrato definitivamente a BotulinFree.com!<br />TORNA ALLA <a href="'. home_url() .'">HOMEPAGE</a>';

	}

	//Register a new user
	function save_newuser($username, $password, $email, $nickname)
	{
		global $wpdb;

		$user_data	= array(
				'user_login'	=> $wpdb->escape( $username ),
				'user_pass'		=> $password,
				'user_email'	=> $email,
				'first_name'	=> $username,
				'nickname'		=> $nickname
			);

		return wp_insert_user($userdata);
	}

	public static function register_reset_password($email, $password) {
		global $wpdb;

		$reset_pass_key = md5(mktime());

		$wpdb->query( $wpdb->prepare("UPDATE `$wpdb->bf_users` SET `reset_pass_key` = %s, 
										`reset_pass_request_time` = NOW(),
										`reset_pass_value` = %s
									WHERE `email` = %s", $reset_pass_key, self::encryptPass($password), $email) );

		return $reset_pass_key;
	}

	public static function confirm_reset_password($email, $reset_pass_key) {
		global $wpdb;

		$result = $wpdb->query( $wpdb->prepare("UPDATE `$wpdb->bf_users` SET `password` = `reset_pass_value`
				WHERE `email` = %s
					AND `reset_pass_key` = %s
					AND `reset_pass_value` IS NOT NULL", $email, $reset_pass_key) ); // TODO: implement the lifetime of the `reset_pass_value`

		if ($result < 1) {
			$user = self::get_user_by_email($email);

			$result = empty($user) ? 0 : 1;
		}

		if ($result > 0)
			$wpdb->query( $wpdb->prepare("UPDATE `$wpdb->bf_users` SET `reset_pass_value` = NULL, `reset_pass_key` = NULL, `reset_pass_request_time` = NULL
				WHERE `email` = %s", $email, $reset_pass_key) );

		return $result > 0;
	}

	public static function send_reset_password_register_confirm($email, $reset_pass_key) {
		
		$email_to = $email;

		$subject = sprintf("%s > %s", get_option("blogname"), __("Conferma modifica password", "wpzoom"));

		$activation_url = home_url("password-reset/?resetpassemailconfirm=${email_to}&key=${reset_pass_key}");

		$home_url = home_url();

		$message = <<< MESSAGE
${subject}

La tua password e' stata modificata con successo. Per confermarla ti preghiamo di cliccare su seguente link:
${activation_url}

Grazie

MESSAGE;
		//var_dump($user->email, $subject, $message, sprintf("From: %s <%s>", get_option("blogname"), get_option("admin_email")));
		//return true;
		mail($email_to, $subject, $message, sprintf("From: %s <%s>", get_option("blogname"), get_option("admin_email")));
	}

	public static function get_user($user_id) {
		global $wpdb;

		$user = $wpdb->get_row( $wpdb->prepare("SELECT * FROM `$wpdb->bf_users` WHERE `id` = %d", $user_id) );

		return $user;
	}

	public static function get_user_by_email($user_email) {
		global $wpdb;

		$user = $wpdb->get_row( $wpdb->prepare("SELECT * FROM `$wpdb->bf_users` WHERE `email` = %d", $user_email) );

		return $user;
	}

	public static function setMessage($msg, $user_type = NULL) {

		if ($user_type === NULL) {
			foreach (self::$user_types as $user_type)
				self::$msg[$user_type] = $msg;
			
			return;
		}

		if (in_array($user_type, self::$user_types))
			self::$msg[$user_type] = $msg;
	}

	public static function setErrorMessage($msg, $user_type = NULL) {

		if ($user_type === NULL) {
			foreach (self::$user_types as $user_type)
				self::$errorMsg[$user_type] = $msg;

			return;
		}

		if (in_array($user_type, self::$user_types))
			self::$errorMsg[$user_type] = $msg;
	}
}